@@ -1,4 +1,4 @@
-# Copyright (c) 2008,2015,2016,2017,2018 MetPy Developers.
+# Copyright (c) 2008,2015,2016,2017,2018,2019 MetPy Developers.
 # Distributed under the terms of the BSD 3-Clause License.
 # SPDX-License-Identifier: BSD-3-Clause
 """Contains a collection of basic calculations.
@@ -25,22 +25,27 @@ from ..xarray import preprocess_xarray
 
 exporter = Exporter(globals())
 
+# The following variables are constants for a standard atmosphere
+t0 = 288. * units.kelvin
+p0 = 1013.25 * units.hPa
+
 
 @exporter.export
 @preprocess_xarray
+@check_units('[speed]', '[speed]')
 def wind_speed(u, v):
     r"""Compute the wind speed from u and v-components.
 
     Parameters
     ----------
-    u : array_like
+    u : `pint.Quantity`
         Wind component in the X (East-West) direction
-    v : array_like
+    v : `pint.Quantity`
         Wind component in the Y (North-South) direction
 
     Returns
     -------
-    wind speed: array_like
+    wind speed: `pint.Quantity`
         The speed of the wind
 
     See Also
@@ -54,21 +59,26 @@ def wind_speed(u, v):
 
 @exporter.export
 @preprocess_xarray
-def wind_direction(u, v):
+@check_units('[speed]', '[speed]')
+def wind_direction(u, v, convention='from'):
     r"""Compute the wind direction from u and v-components.
 
     Parameters
     ----------
-    u : array_like
+    u : `pint.Quantity`
         Wind component in the X (East-West) direction
-    v : array_like
+    v : `pint.Quantity`
         Wind component in the Y (North-South) direction
+    convention : str
+        Convention to return direction. 'from' returns the direction the wind is coming from
+        (meteorological convention). 'to' returns the direction the wind is going towards
+        (oceanographic convention). Default is 'from'.
 
     Returns
     -------
     direction: `pint.Quantity`
-        The direction of the wind in interval [0, 360] degrees, specified as the direction from
-        which it is blowing, with 360 being North.
+        The direction of the wind in interval [0, 360] degrees, with 360 being North, with the
+        direction defined by the convention kwarg.
 
     See Also
     --------
@@ -83,6 +93,13 @@ def wind_direction(u, v):
     wdir = 90. * units.deg - np.arctan2(-v, -u)
     origshape = wdir.shape
     wdir = atleast_1d(wdir)
+
+    # Handle oceanographic convection
+    if convention == 'to':
+        wdir -= 180 * units.deg
+    elif convention not in ('to', 'from'):
+        raise KeyError('Invalid kwarg for "convention". Valid options are "from" or "to".')
+
     wdir[wdir <= 0] += 360. * units.deg
     # Need to be able to handle array-like u and v (with or without units)
     # np.any check required for legacy numpy which treats 0-d False boolean index as zero
@@ -94,20 +111,21 @@ def wind_direction(u, v):
 
 @exporter.export
 @preprocess_xarray
+@check_units('[speed]')
 def wind_components(speed, wdir):
     r"""Calculate the U, V wind vector components from the speed and direction.
 
     Parameters
     ----------
-    speed : array_like
+    speed : `pint.Quantity`
         The wind speed (magnitude)
-    wdir : array_like
+    wdir : `pint.Quantity`
         The wind direction, specified as the direction from which the wind is
         blowing (0-2 pi radians or 0-360 degrees), with 360 degrees being North.
 
     Returns
     -------
-    u, v : tuple of array_like
+    u, v : tuple of `pint.Quantity`
         The wind components in the X (East-West) and Y (North-South)
         directions, respectively.
 
@@ -241,14 +259,17 @@ def windchill(temperature, speed, face_level_winds=False, mask_undefined=True):
 def heat_index(temperature, rh, mask_undefined=True):
     r"""Calculate the Heat Index from the current temperature and relative humidity.
 
-    The implementation uses the formula outlined in [Rothfusz1990]_. This equation is a
+    The implementation uses the formula outlined in [Rothfusz1990]_, which is a
     multi-variable least-squares regression of the values obtained in [Steadman1979]_.
+    Additional conditional corrections are applied to match what the National
+    Weather Service operationally uses. See Figure 3 of [Anderson2013]_ for a
+    depiction of this algorithm and further discussion.
 
     Parameters
     ----------
     temperature : `pint.Quantity`
         Air temperature
-    rh : array_like
+    rh : `pint.Quantity`
         The relative humidity expressed as a unitless ratio in the range [0, 1].
         Can also pass a percentage if proper units are attached.
 
@@ -261,43 +282,85 @@ def heat_index(temperature, rh, mask_undefined=True):
     ----------------
     mask_undefined : bool, optional
         A flag indicating whether a masked array should be returned with
-        values where heat index is undefined masked.  These are values where
-        the temperature < 80F or relative humidity < 40 percent. Defaults
-        to `True`.
+        values masked where the temperature < 80F. Defaults to `True`.
 
     See Also
     --------
     windchill
 
     """
+    temperature = atleast_1d(temperature)
+    rh = atleast_1d(rh)
+    # assign units to rh if they currently are not present
+    if not hasattr(rh, 'units'):
+        rh = rh * units.dimensionless
     delta = temperature.to(units.degF) - 0. * units.degF
     rh2 = rh * rh
     delta2 = delta * delta
 
-    # Calculate the Heat Index -- constants converted for RH in [0, 1]
-    hi = (-42.379 * units.degF
-          + 2.04901523 * delta
-          + 1014.333127 * units.delta_degF * rh
-          - 22.475541 * delta * rh
-          - 6.83783e-3 / units.delta_degF * delta2
-          - 5.481717e2 * units.delta_degF * rh2
-          + 1.22874e-1 / units.delta_degF * delta2 * rh
-          + 8.5282 * delta * rh2
-          - 1.99e-2 / units.delta_degF * delta2 * rh2)
+    # Simplifed Heat Index -- constants converted for RH in [0, 1]
+    a = -10.3 * units.degF + 1.1 * delta + 4.7 * units.delta_degF * rh
+
+    # More refined Heat Index -- constants converted for RH in [0, 1]
+    b = (-42.379 * units.degF
+         + 2.04901523 * delta
+         + 1014.333127 * units.delta_degF * rh
+         - 22.475541 * delta * rh
+         - 6.83783e-3 / units.delta_degF * delta2
+         - 5.481717e2 * units.delta_degF * rh2
+         + 1.22874e-1 / units.delta_degF * delta2 * rh
+         + 8.5282 * delta * rh2
+         - 1.99e-2 / units.delta_degF * delta2 * rh2)
+
+    # Create return heat index
+    hi = np.full(np.shape(temperature), np.nan) * units.degF
+    # Retain masked status of temperature with resulting heat index
+    if hasattr(temperature, 'mask'):
+        hi = masked_array(hi)
+
+    # If T <= 40F, Heat Index is T
+    sel = (temperature <= 40. * units.degF)
+    if np.any(sel):
+        hi[sel] = temperature[sel].to(units.degF)
+
+    # If a < 79F and hi is unset, Heat Index is a
+    sel = (a < 79. * units.degF) & np.isnan(hi)
+    if np.any(sel):
+        hi[sel] = a[sel]
+
+    # Use b now for anywhere hi has yet to be set
+    sel = np.isnan(hi)
+    if np.any(sel):
+        hi[sel] = b[sel]
+
+    # Adjustment for RH <= 13% and 80F <= T <= 112F
+    sel = ((rh <= 13. * units.percent) & (temperature >= 80. * units.degF)
+           & (temperature <= 112. * units.degF))
+    if np.any(sel):
+        rh15adj = ((13. - rh * 100.) / 4.
+                   * ((17. * units.delta_degF - np.abs(delta - 95. * units.delta_degF))
+                      / 17. * units.delta_degF) ** 0.5)
+        hi[sel] = hi[sel] - rh15adj[sel]
+
+    # Adjustment for RH > 85% and 80F <= T <= 87F
+    sel = ((rh > 85. * units.percent) & (temperature >= 80. * units.degF)
+           & (temperature <= 87. * units.degF))
+    if np.any(sel):
+        rh85adj = 0.02 * (rh * 100. - 85.) * (87. * units.delta_degF - delta)
+        hi[sel] = hi[sel] + rh85adj[sel]
 
     # See if we need to mask any undefined values
     if mask_undefined:
-        mask = np.array((temperature < 80. * units.degF) | (rh < 40 * units.percent))
+        mask = np.array(temperature < 80. * units.degF)
         if mask.any():
             hi = masked_array(hi, mask=mask)
-
     return hi
 
 
 @exporter.export
 @preprocess_xarray
 @check_units(temperature='[temperature]', speed='[speed]')
-def apparent_temperature(temperature, rh, speed, face_level_winds=False):
+def apparent_temperature(temperature, rh, speed, face_level_winds=False, mask_undefined=True):
     r"""Calculate the current apparent temperature.
 
     Calculates the current apparent temperature based on the wind chill or heat index
@@ -318,6 +381,13 @@ def apparent_temperature(temperature, rh, speed, face_level_winds=False):
         A flag indicating whether the wind speeds were measured at facial
         level instead of 10m, thus requiring a correction.  Defaults to
         `False`.
+    mask_undefined : bool, optional
+        A flag indicating whether a masked array should be returned with
+        values where wind chill or heat_index is undefined masked. For wind
+        chill, these are values where the temperature > 50F or
+        wind speed <= 3 miles per hour. For heat index, these are values
+        where the temperature < 80F.
+        Defaults to `True`.
 
     Returns
     -------
@@ -335,6 +405,7 @@ def apparent_temperature(temperature, rh, speed, face_level_winds=False):
     rh = atleast_1d(rh)
     speed = atleast_1d(speed)
 
+    # NB: mask_defined=True is needed to know where computed values exist
     wind_chill_temperature = windchill(temperature, speed, face_level_winds=face_level_winds,
                                        mask_undefined=True).to(temperature.units)
 
@@ -342,26 +413,33 @@ def apparent_temperature(temperature, rh, speed, face_level_winds=False):
                                         mask_undefined=True).to(temperature.units)
 
     # Combine the heat index and wind chill arrays (no point has a value in both)
-    app_temperature = np.ma.where(masked_array(wind_chill_temperature).mask,
-                                  heat_index_temperature,
-                                  wind_chill_temperature)
+    # NB: older numpy.ma.where does not return a masked array
+    app_temperature = masked_array(
+        np.ma.where(masked_array(wind_chill_temperature).mask,
+                    heat_index_temperature.to(temperature.units),
+                    wind_chill_temperature.to(temperature.units)
+                    ), temperature.units)
+
+    # If mask_undefined is False, then set any masked values to the temperature
+    if not mask_undefined:
+        app_temperature[app_temperature.mask] = temperature[app_temperature.mask]
+
+    # If no values are masked and provided temperature does not have a mask
+    # we should return a non-masked array
+    if not np.any(app_temperature.mask) and not hasattr(temperature, 'mask'):
+        app_temperature = np.array(app_temperature.m) * temperature.units
 
     if is_not_scalar:
-        # Fill in missing areas where neither wind chill or heat index are applicable with the
-        # ambient temperature.
-        app_temperature[app_temperature.mask] = temperature[app_temperature.mask]
-        return np.array(app_temperature) * temperature.units
+        return app_temperature
     else:
-        if app_temperature.mask:
-            app_temperature = temperature.m
-        return atleast_1d(app_temperature)[0] * temperature.units
+        return atleast_1d(app_temperature)[0]
 
 
 @exporter.export
 @preprocess_xarray
 @check_units('[pressure]')
 def pressure_to_height_std(pressure):
-    r"""Convert pressure data to heights using the U.S. standard atmosphere.
+    r"""Convert pressure data to heights using the U.S. standard atmosphere [NOAA1976]_.
 
     The implementation uses the formula outlined in [Hobbs1977]_ pg.60-61.
 
@@ -380,9 +458,7 @@ def pressure_to_height_std(pressure):
     .. math:: Z = \frac{T_0}{\Gamma}[1-\frac{p}{p_0}^\frac{R\Gamma}{g}]
 
     """
-    t0 = 288. * units.kelvin
     gamma = 6.5 * units('K/km')
-    p0 = 1013.25 * units.mbar
     return (t0 / gamma) * (1 - (pressure / p0).to('dimensionless')**(
         mpconsts.Rd * gamma / mpconsts.g))
 
@@ -393,10 +469,19 @@ def pressure_to_height_std(pressure):
 def height_to_geopotential(height):
     r"""Compute geopotential for a given height.
 
+    Calculates the geopotential from height using the following formula, which is derived from
+    the definition of geopotential as given in [Hobbs2006]_ Pg. 69 Eq 3.21:
+
+    .. math:: \Phi = G m_e \left( \frac{1}{R_e} - \frac{1}{R_e + z}\right)
+
+    (where :math:`\Phi` is geopotential, :math:`z` is height, :math:`R_e` is average Earth
+    radius, :math:`G` is the (universal) gravitational constant, and :math:`m_e` is the
+    approximate mass of Earth.)
+
     Parameters
     ----------
     height : `pint.Quantity`
-        Height above sea level (array_like)
+        Height above sea level
 
     Returns
     -------
@@ -405,25 +490,19 @@ def height_to_geopotential(height):
 
     Examples
     --------
-    >>> from metpy.constants import g, G, me, Re
     >>> import metpy.calc
     >>> from metpy.units import units
-    >>> height = np.linspace(0,10000, num = 11) * units.m
+    >>> height = np.linspace(0, 10000, num=11) * units.m
     >>> geopot = metpy.calc.height_to_geopotential(height)
     >>> geopot
-    <Quantity([     0.           9817.46806283  19631.85526579  29443.16305888
-    39251.39289118  49056.54621087  58858.62446525  68657.62910064
-    78453.56156253  88246.42329545  98036.21574306], 'meter ** 2 / second ** 2')>
-
-    Notes
-    -----
-    Derived from definition of geopotential in [Hobbs2006]_ pg.14 Eq.1.8.
+    <Quantity([     0.           9817.46806283  19631.85526579  29443.16305887
+    39251.39289118  49056.54621087  58858.62446524  68657.62910064
+    78453.56156252  88246.42329544  98036.21574305], 'meter ** 2 / second ** 2')>
 
     """
-    # Calculate geopotential
-    geopot = mpconsts.G * mpconsts.me * ((1 / mpconsts.Re) - (1 / (mpconsts.Re + height)))
-
-    return geopot
+    # Direct implementation of formula from Hobbs yields poor numerical results (see
+    # gh-1075), so was replaced with algebraic equivalent.
+    return (mpconsts.G * mpconsts.me / mpconsts.Re) * (height / (mpconsts.Re + height))
 
 
 @exporter.export
@@ -431,10 +510,19 @@ def height_to_geopotential(height):
 def geopotential_to_height(geopot):
     r"""Compute height from a given geopotential.
 
+    Calculates the height from geopotential using the following formula, which is derived from
+    the definition of geopotential as given in [Hobbs2006]_ Pg. 69 Eq 3.21:
+
+    .. math:: z = \frac{1}{\frac{1}{R_e} - \frac{\Phi}{G m_e}} - R_e
+
+    (where :math:`\Phi` is geopotential, :math:`z` is height, :math:`R_e` is average Earth
+    radius, :math:`G` is the (universal) gravitational constant, and :math:`m_e` is the
+    approximate mass of Earth.)
+
     Parameters
     ----------
     geopotential : `pint.Quantity`
-        Geopotential (array_like)
+        Geopotential
 
     Returns
     -------
@@ -443,36 +531,31 @@ def geopotential_to_height(geopot):
 
     Examples
     --------
-    >>> from metpy.constants import g, G, me, Re
     >>> import metpy.calc
     >>> from metpy.units import units
-    >>> height = np.linspace(0,10000, num = 11) * units.m
+    >>> height = np.linspace(0, 10000, num=11) * units.m
     >>> geopot = metpy.calc.height_to_geopotential(height)
     >>> geopot
-    <Quantity([     0.           9817.46806283  19631.85526579  29443.16305888
-    39251.39289118  49056.54621087  58858.62446525  68657.62910064
-    78453.56156253  88246.42329545  98036.21574306], 'meter ** 2 / second ** 2')>
+    <Quantity([     0.           9817.46806283  19631.85526579  29443.16305887
+    39251.39289118  49056.54621087  58858.62446524  68657.62910064
+    78453.56156252  88246.42329544  98036.21574305], 'meter ** 2 / second ** 2')>
     >>> height = metpy.calc.geopotential_to_height(geopot)
     >>> height
     <Quantity([     0.   1000.   2000.   3000.   4000.   5000.   6000.   7000.   8000.
     9000.  10000.], 'meter')>
 
-    Notes
-    -----
-    Derived from definition of geopotential in [Hobbs2006]_ pg.14 Eq.1.8.
-
     """
-    # Calculate geopotential
-    height = (((1 / mpconsts.Re) - (geopot / (mpconsts.G * mpconsts.me))) ** -1) - mpconsts.Re
-
-    return height
+    # Direct implementation of formula from Hobbs yields poor numerical results (see
+    # gh-1075), so was replaced with algebraic equivalent.
+    scaled = geopot * mpconsts.Re
+    return scaled * mpconsts.Re / (mpconsts.G * mpconsts.me - scaled)
 
 
 @exporter.export
 @preprocess_xarray
 @check_units('[length]')
 def height_to_pressure_std(height):
-    r"""Convert height data to pressures using the U.S. standard atmosphere.
+    r"""Convert height data to pressures using the U.S. standard atmosphere [NOAA1976]_.
 
     The implementation inverts the formula outlined in [Hobbs1977]_ pg.60-61.
 
@@ -491,9 +574,7 @@ def height_to_pressure_std(height):
     .. math:: p = p_0 e^{\frac{g}{R \Gamma} \text{ln}(1-\frac{Z \Gamma}{T_0})}
 
     """
-    t0 = 288. * units.kelvin
     gamma = 6.5 * units('K/km')
-    p0 = 1013.25 * units.mbar
     return p0 * (1 - (gamma / t0) * height) ** (mpconsts.g / (mpconsts.Rd * gamma))
 
 
@@ -525,7 +606,7 @@ def coriolis_parameter(latitude):
 def add_height_to_pressure(pressure, height):
     r"""Calculate the pressure at a certain height above another pressure level.
 
-    This assumes a standard atmosphere.
+    This assumes a standard atmosphere [NOAA1976]_.
 
     Parameters
     ----------
@@ -554,7 +635,7 @@ def add_height_to_pressure(pressure, height):
 def add_pressure_to_height(height, pressure):
     r"""Calculate the height at a certain pressure above another height.
 
-    This assumes a standard atmosphere.
+    This assumes a standard atmosphere [NOAA1976]_.
 
     Parameters
     ----------
@@ -773,6 +854,159 @@ def smooth_n_point(scalar_grid, n=5, passes=1):
     return smooth_grid
 
 
+@exporter.export
+@preprocess_xarray
+@check_units('[pressure]', '[length]')
+def altimeter_to_station_pressure(altimeter_value, height):
+    r"""Convert the altimeter measurement to station pressure.
+
+    This function is useful for working with METARs since they do not provide
+    altimeter values, but not sea-level pressure or station pressure.
+    The following definitions of altimeter setting and station pressure
+    are taken from [Smithsonian1951]_ Altimeter setting is the
+    pressure value to which an aircraft altimeter scale is set so that it will
+    indicate the altitude above mean sea-level of an aircraft on the ground at the
+    location for which the value is determined. It assumes a standard atmosphere [NOAA1976]_.
+    Station pressure is the atmospheric pressure at the designated station elevation.
+    Finding the station pressure can be helpful for calculating sea-level pressure
+    or other parameters.
+
+    Parameters
+    ----------
+    altimeter_value : `pint.Quantity`
+        The altimeter setting value as defined by the METAR or other observation,
+        which can be measured in either inches of mercury (in. Hg) or millibars (mb)
+    height: `pint.Quantity`
+        Elevation of the station measuring pressure.
+
+    Returns
+    -------
+    `pint.Quantity`
+        The station pressure in hPa or in. Hg, which can be used to calculate sea-level
+        pressure
+
+    See Also
+    --------
+    altimeter_to_sea_level_pressure
+
+    Notes
+    -----
+    This function is implemented using the following equations from the
+    Smithsonian Handbook (1951) p. 269
+
+    Equation 1:
+     .. math:: A_{mb} = (p_{mb} - 0.3)F
+
+    Equation 3:
+     .. math::  F = \left [1 + \left(\frac{p_{0}^n a}{T_{0}} \right)
+                   \frac{H_{b}}{p_{1}^n} \right ] ^ \frac{1}{n}
+
+    Where
+
+    :math:`p_{0}` = standard sea-level pressure = 1013.25 mb
+
+    :math:`p_{1} = p_{mb} - 0.3` when :math:`p_{0} = 1013.25 mb`
+
+    gamma = lapse rate in [NOAA1976]_ standard atmosphere below the isothermal layer
+    :math:`6.5^{\circ}C. km.^{-1}`
+
+    :math:`t_{0}` = standard sea-level temperature 288 K
+
+    :math:`H_{b} =` station elevation in meters (elevation for which station
+      pressure is given)
+
+    :math:`n = \frac{a R_{d}}{g} = 0.190284` where :math:`R_{d}` is the gas
+      constant for dry air
+
+    And solving for :math:`p_{mb}` results in the equation below, which is used to
+    calculate station pressure :math:`(p_{mb})`
+
+    .. math:: p_{mb} = \left [A_{mb} ^ n - \left (\frac{p_{0} a H_{b}}{T_0}
+                       \right) \right] ^ \frac{1}{n} + 0.3
+
+    """
+    # Gamma Value for this case
+    gamma = 0.0065 * units('K/m')
+
+    # N-Value
+    n = (mpconsts.Rd * gamma / mpconsts.g).to_base_units()
+
+    return ((altimeter_value ** n
+             - ((p0.to(altimeter_value.units) ** n * gamma * height) / t0)) ** (1 / n)
+            + 0.3 * units.hPa)
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[pressure]', '[length]', '[temperature]')
+def altimeter_to_sea_level_pressure(altimeter_value, height, temperature):
+    r"""Convert the altimeter setting to sea-level pressure.
+
+    This function is useful for working with METARs since most provide
+    altimeter values, but not sea-level pressure, which is often plotted
+    on surface maps. The following definitions of altimeter setting, station pressure, and
+    sea-level pressure are taken from [Smithsonian1951]_
+    Altimeter setting is the pressure value to which an aircraft altimeter scale
+    is set so that it will indicate the altitude above mean sea-level of an aircraft
+    on the ground at the location for which the value is determined. It assumes a standard
+    atmosphere. Station pressure is the atmospheric pressure at the designated station
+    elevation. Sea-level pressure is a pressure value obtained by the theoretical reduction
+    of barometric pressure to sea level. It is assumed that atmosphere extends to sea level
+    below the station and that the properties of the atmosphere are related to conditions
+    observed at the station. This value is recorded by some surface observation stations,
+    but not all. If the value is recorded, it can be found in the remarks section. Finding
+    the sea-level pressure is helpful for plotting purposes and different calculations.
+
+    Parameters
+    ----------
+    altimeter_value : 'pint.Quantity'
+        The altimeter setting value is defined by the METAR or other observation,
+        with units of inches of mercury (in Hg) or millibars (hPa)
+    height  : 'pint.Quantity'
+        Elevation of the station measuring pressure. Often times measured in meters
+    temperature : 'pint.Quantity'
+        Temperature at the station
+
+    Returns
+    -------
+    'pint.Quantity'
+        The sea-level pressure in hPa and makes pressure values easier to compare
+        between different stations
+
+    See Also
+    --------
+    altimeter_to_station_pressure
+
+    Notes
+    -----
+    This function is implemented using the following equations from Wallace and Hobbs (1977)
+
+    Equation 2.29:
+     .. math::
+       \Delta z = Z_{2} - Z_{1}
+       = \frac{R_{d} \bar T_{v}}{g_0}ln\left(\frac{p_{1}}{p_{2}}\right)
+       = \bar H ln \left (\frac {p_{1}}{p_{2}} \right)
+
+    Equation 2.31:
+     .. math::
+       p_{0} = p_{g}exp \left(\frac{Z_{g}}{\bar H} \right) \\
+       = p_{g}exp \left(\frac{g_{0}Z_{g}}{R_{d}\bar T_{v}} \right)
+
+    Then by substituting :math:`Delta_{Z}` for :math:`Z_{g}` in Equation 2.31:
+     .. math:: p_{sea_level} = p_{station} exp\left(\frac{\Delta z}{H}\right)
+
+    where :math:`Delta_{Z}` is the elevation in meters and :math:`H = \frac{R_{d}T}{g}`
+
+    """
+    # Calculate the station pressure using function altimeter_to_station_pressure()
+    psfc = altimeter_to_station_pressure(altimeter_value, height)
+
+    # Calculate the scale height
+    h = mpconsts.Rd * temperature / mpconsts.g
+
+    return psfc * np.exp(height / h)
+
+
 def _check_radians(value, max_radians=2 * np.pi):
     """Input validation of values that could be in degrees instead of radians.
 
