@@ -1,4 +1,4 @@
-# Copyright (c) 2008,2015,2016,2017,2018 MetPy Developers.
+# Copyright (c) 2008,2015,2016,2017,2018,2019 MetPy Developers.
 # Distributed under the terms of the BSD 3-Clause License.
 # SPDX-License-Identifier: BSD-3-Clause
 """Contains a collection of thermodynamic calculations."""
@@ -15,6 +15,7 @@ from .tools import (_greater_or_close, _less_or_close, find_bounding_indices,
                     find_intersections, first_derivative, get_layer)
 from .. import constants as mpconsts
 from ..cbook import broadcast_indices
+from ..deprecation import metpyDeprecation
 from ..interpolate.one_dimension import interpolate_1d
 from ..package_tools import Exporter
 from ..units import atleast_1d, check_units, concatenate, units
@@ -37,14 +38,14 @@ def relative_humidity_from_dewpoint(temperature, dewpt):
     Parameters
     ----------
     temperature : `pint.Quantity`
-        The temperature
-    dew point : `pint.Quantity`
-        The dew point temperature
+        air temperature
+    dewpoint : `pint.Quantity`
+        dewpoint temperature
 
     Returns
     -------
     `pint.Quantity`
-        The relative humidity
+        relative humidity
 
     See Also
     --------
@@ -72,9 +73,10 @@ def exner_function(pressure, reference_pressure=mpconsts.P0):
     Parameters
     ----------
     pressure : `pint.Quantity`
-        The total atmospheric pressure
+        total atmospheric pressure
     reference_pressure : `pint.Quantity`, optional
-        The reference pressure against which to calculate the Exner function, defaults to P0
+        The reference pressure against which to calculate the Exner function, defaults to
+        metpy.constants.P0
 
     Returns
     -------
@@ -102,9 +104,9 @@ def potential_temperature(pressure, temperature):
     Parameters
     ----------
     pressure : `pint.Quantity`
-        The total atmospheric pressure
+        total atmospheric pressure
     temperature : `pint.Quantity`
-        The temperature
+        air temperature
 
     Returns
     -------
@@ -144,9 +146,9 @@ def temperature_from_potential_temperature(pressure, theta):
     Parameters
     ----------
     pressure : `pint.Quantity`
-        The total atmospheric pressure
+        total atmospheric pressure
     theta : `pint.Quantity`
-        The potential temperature
+        potential temperature
 
     Returns
     -------
@@ -203,8 +205,7 @@ def dry_lapse(pressure, temperature, ref_pressure=None):
 
     See Also
     --------
-    moist_lapse : Calculate parcel temperature assuming liquid saturation
-                  processes
+    moist_lapse : Calculate parcel temperature assuming liquid saturation processes
     parcel_profile : Calculate complete parcel profile
     potential_temperature
 
@@ -263,7 +264,7 @@ def moist_lapse(pressure, temperature, ref_pressure=None):
         frac = ((mpconsts.Rd * t + mpconsts.Lv * rs)
                 / (mpconsts.Cp_d + (mpconsts.Lv * mpconsts.Lv * rs * mpconsts.epsilon
                                     / (mpconsts.Rd * t * t)))).to('kelvin')
-        return frac / p
+        return (frac / p).magnitude
 
     if ref_pressure is None:
         ref_pressure = pressure[0]
@@ -286,14 +287,14 @@ def moist_lapse(pressure, temperature, ref_pressure=None):
 
     if ref_pressure > pressure.min():
         # Integrate downward in pressure
-        pres_down = np.append(ref_pressure, pressure[(ref_pres_idx - 1)::-1])
-        trace_down = si.odeint(dt, temperature.squeeze(), pres_down.squeeze())
+        pres_down = np.append(ref_pressure.m, pressure[(ref_pres_idx - 1)::-1].m)
+        trace_down = si.odeint(dt, temperature.m.squeeze(), pres_down.squeeze())
         ret_temperatures = np.concatenate((ret_temperatures, trace_down[:0:-1]))
 
     if ref_pressure < pressure.max():
         # Integrate upward in pressure
-        pres_up = np.append(ref_pressure, pressure[ref_pres_idx:])
-        trace_up = si.odeint(dt, temperature.squeeze(), pres_up.squeeze())
+        pres_up = np.append(ref_pressure.m, pressure[ref_pres_idx:].m)
+        trace_up = si.odeint(dt, temperature.m.squeeze(), pres_up.squeeze())
         ret_temperatures = np.concatenate((ret_temperatures, trace_up[1:]))
 
     if pres_decreasing:
@@ -318,12 +319,14 @@ def lcl(pressure, temperature, dewpt, max_iters=50, eps=1e-5):
     temperature : `pint.Quantity`
         The starting temperature
     dewpt : `pint.Quantity`
-        The starting dew point
+        The starting dewpoint
 
     Returns
     -------
-    `(pint.Quantity, pint.Quantity)`
-        The LCL pressure and temperature
+    `pint.Quantity`
+        The LCL pressure
+    `pint.Quantity`
+        The LCL temperature
 
     Other Parameters
     ----------------
@@ -341,7 +344,7 @@ def lcl(pressure, temperature, dewpt, max_iters=50, eps=1e-5):
     This function is implemented using an iterative approach to solve for the
     LCL. The basic algorithm is:
 
-    1. Find the dew point from the LCL pressure and starting mixing ratio
+    1. Find the dewpoint from the LCL pressure and starting mixing ratio
     2. Find the LCL pressure from the starting temperature and dewpoint
     3. Iterate until convergence
 
@@ -356,13 +359,14 @@ def lcl(pressure, temperature, dewpt, max_iters=50, eps=1e-5):
     fp = so.fixed_point(_lcl_iter, pressure.m, args=(pressure.m, w, temperature),
                         xtol=eps, maxiter=max_iters)
     lcl_p = fp * pressure.units
-    return lcl_p, dewpoint(vapor_pressure(lcl_p, w))
+    return lcl_p, dewpoint(vapor_pressure(lcl_p, w)).to(temperature.units)
 
 
 @exporter.export
 @preprocess_xarray
 @check_units('[pressure]', '[temperature]', '[temperature]', '[temperature]')
-def lfc(pressure, temperature, dewpt, parcel_temperature_profile=None):
+def lfc(pressure, temperature, dewpt, parcel_temperature_profile=None, dewpt_start=None,
+        which='top'):
     r"""Calculate the level of free convection (LFC).
 
     This works by finding the first intersection of the ideal parcel path and
@@ -375,15 +379,23 @@ def lfc(pressure, temperature, dewpt, parcel_temperature_profile=None):
     temperature : `pint.Quantity`
         The temperature at the levels given by `pressure`
     dewpt : `pint.Quantity`
-        The dew point at the levels given by `pressure`
+        The dewpoint at the levels given by `pressure`
     parcel_temperature_profile: `pint.Quantity`, optional
         The parcel temperature profile from which to calculate the LFC. Defaults to the
         surface parcel profile.
+    dewpt_start: `pint.Quantity`, optional
+        The dewpoint of the parcel for which to calculate the LFC. Defaults to the surface
+        dewpoint.
+    which: str, optional
+        Pick which LFC to return. Options are 'top', 'bottom', and 'all'.
+        Default is the 'top' (lowest pressure) LFC.
 
     Returns
     -------
     `pint.Quantity`
-        The LFC pressure and temperature
+        The LFC pressure, or array of same if which='all'
+    `pint.Quantity`
+        The LFC temperature, or array of same if which='all'
 
     See Also
     --------
@@ -394,13 +406,23 @@ def lfc(pressure, temperature, dewpt, parcel_temperature_profile=None):
     if parcel_temperature_profile is None:
         new_stuff = parcel_profile_with_lcl(pressure, temperature, dewpt)
         pressure, temperature, _, parcel_temperature_profile = new_stuff
-        temperature = temperature.to('degC')
-        parcel_temperature_profile = parcel_temperature_profile.to('degC')
+        parcel_temperature_profile = parcel_temperature_profile.to(temperature.units)
 
-    # The parcel profile and data have the same first data point, so we ignore
-    # that point to get the real first intersection for the LFC calculation.
-    x, y = find_intersections(pressure[1:], parcel_temperature_profile[1:],
-                              temperature[1:], direction='increasing')
+    if dewpt_start is None:
+        dewpt_start = dewpt[0]
+
+    # The parcel profile and data may have the same first data point.
+    # If that is the case, ignore that point to get the real first
+    # intersection for the LFC calculation. Use logarithmic interpolation.
+    if np.isclose(parcel_temperature_profile[0].m, temperature[0].m):
+        x, y = find_intersections(pressure[1:], parcel_temperature_profile[1:],
+                                  temperature[1:], direction='increasing', log_x=True)
+    else:
+        x, y = find_intersections(pressure, parcel_temperature_profile,
+                                  temperature, direction='increasing', log_x=True)
+
+    # Compute LCL for this parcel for future comparisons
+    this_lcl = lcl(pressure[0], parcel_temperature_profile[0], dewpt_start)
 
     # The LFC could:
     # 1) Not exist
@@ -409,25 +431,51 @@ def lfc(pressure, temperature, dewpt, parcel_temperature_profile=None):
 
     # LFC does not exist or is LCL
     if len(x) == 0:
-        if np.all(_less_or_close(parcel_temperature_profile, temperature)):
+        # Is there any positive area above the LCL?
+        mask = pressure < this_lcl[0]
+        if np.all(_less_or_close(parcel_temperature_profile[mask], temperature[mask])):
             # LFC doesn't exist
-            return np.nan * pressure.units, np.nan * temperature.units
+            x, y = np.nan * pressure.units, np.nan * temperature.units
         else:  # LFC = LCL
-            x, y = lcl(pressure[0], temperature[0], dewpt[0])
-            return x, y
+            x, y = this_lcl
+        return x, y
 
-    # LFC exists and is not LCL. Make sure it is above the LCL.
+    # LFC exists. Make sure it is no lower than the LCL
     else:
-        idx = x < lcl(pressure[0], temperature[0], dewpt[0])[0]
-        x = x[idx]
-        y = y[idx]
-        return x[0], y[0]
+        idx = x < this_lcl[0]
+        # LFC height < LCL height, so set LFC = LCL
+        if not any(idx):
+            el_pres, _ = find_intersections(pressure[1:], parcel_temperature_profile[1:],
+                                            temperature[1:], direction='decreasing',
+                                            log_x=True)
+            if np.min(el_pres) > this_lcl[0]:
+                x, y = np.nan * pressure.units, np.nan * temperature.units
+            else:
+                x, y = this_lcl
+            return x, y
+        # Otherwise, find all LFCs that exist above the LCL
+        # What is returned depends on which flag as described in the docstring
+        else:
+            return _multiple_el_lfc_options(x, y, idx, which)
+
+
+def _multiple_el_lfc_options(intersect_pressures, intersect_temperatures, valid_x,
+                             which):
+    """Choose which ELs and LFCs to return from a sounding."""
+    p_list, t_list = intersect_pressures[valid_x], intersect_temperatures[valid_x]
+    if which == 'all':
+        x, y = p_list, t_list
+    elif which == 'bottom':
+        x, y = p_list[0], t_list[0]
+    elif which == 'top':
+        x, y = p_list[-1], t_list[-1]
+    return x, y
 
 
 @exporter.export
 @preprocess_xarray
 @check_units('[pressure]', '[temperature]', '[temperature]', '[temperature]')
-def el(pressure, temperature, dewpt, parcel_temperature_profile=None):
+def el(pressure, temperature, dewpt, parcel_temperature_profile=None, which='top'):
     r"""Calculate the equilibrium level.
 
     This works by finding the last intersection of the ideal parcel path and
@@ -437,19 +485,24 @@ def el(pressure, temperature, dewpt, parcel_temperature_profile=None):
     Parameters
     ----------
     pressure : `pint.Quantity`
-        The atmospheric pressure
+        The atmospheric pressure profile
     temperature : `pint.Quantity`
         The temperature at the levels given by `pressure`
     dewpt : `pint.Quantity`
-        The dew point at the levels given by `pressure`
+        The dewpoint at the levels given by `pressure`
     parcel_temperature_profile: `pint.Quantity`, optional
         The parcel temperature profile from which to calculate the EL. Defaults to the
         surface parcel profile.
+    which: str, optional
+        Pick which EL to return. Options are 'top', 'bottom', and 'all'.
+        Default is the 'top' (lowest pressure) EL.
 
     Returns
     -------
-    `pint.Quantity, pint.Quantity`
-        The EL pressure and temperature
+    `pint.Quantity`
+        The EL pressure, or array of same if which='all'
+    `pint.Quantity`
+        The EL temperature, or array of same if which='all'
 
     See Also
     --------
@@ -460,17 +513,20 @@ def el(pressure, temperature, dewpt, parcel_temperature_profile=None):
     if parcel_temperature_profile is None:
         new_stuff = parcel_profile_with_lcl(pressure, temperature, dewpt)
         pressure, temperature, _, parcel_temperature_profile = new_stuff
-        temperature = temperature.to('degC')
-        parcel_temperature_profile = parcel_temperature_profile.to('degC')
+        parcel_temperature_profile = parcel_temperature_profile.to(temperature.units)
 
     # If the top of the sounding parcel is warmer than the environment, there is no EL
     if parcel_temperature_profile[-1] > temperature[-1]:
         return np.nan * pressure.units, np.nan * temperature.units
 
-    # Otherwise the last intersection (as long as there is one) is the EL
-    x, y = find_intersections(pressure[1:], parcel_temperature_profile[1:], temperature[1:])
-    if len(x) > 0:
-        return x[-1], y[-1]
+    # Interpolate in log space to find the appropriate pressure - units have to be stripped
+    # and reassigned to allow np.log() to function properly.
+    x, y = find_intersections(pressure[1:], parcel_temperature_profile[1:], temperature[1:],
+                              direction='decreasing', log_x=True)
+    lcl_p, _ = lcl(pressure[0], temperature[0], dewpt[0])
+    idx = x < lcl_p
+    if len(x) > 0 and x[-1] < lcl_p:
+        return _multiple_el_lfc_options(x, y, idx, which)
     else:
         return np.nan * pressure.units, np.nan * temperature.units
 
@@ -488,12 +544,12 @@ def parcel_profile(pressure, temperature, dewpt):
     Parameters
     ----------
     pressure : `pint.Quantity`
-        The atmospheric pressure level(s) of interest. The first entry should be the starting
-        point pressure.
+        The atmospheric pressure level(s) of interest. This array must be from
+        high to low pressure.
     temperature : `pint.Quantity`
         The starting temperature
     dewpt : `pint.Quantity`
-        The starting dew point
+        The starting dewpoint
 
     Returns
     -------
@@ -523,14 +579,14 @@ def parcel_profile_with_lcl(pressure, temperature, dewpt):
     Parameters
     ----------
     pressure : `pint.Quantity`
-        The atmospheric pressure level(s) of interest. The first entry should be the starting
-        point pressure.
+        The atmospheric pressure level(s) of interest. This array must be from
+        high to low pressure.
     temperature : `pint.Quantity`
-        The atmospheric temperature at the levels in `pressure`. The first entry should be the
-        starting point temperature.
+        The atmospheric temperature at the levels in `pressure`. The first entry should be at
+        the same level as the first `pressure` data point.
     dewpt : `pint.Quantity`
-        The atmospheric dew point at the levels in `pressure`. The first entry should be the
-        starting dew point.
+        The atmospheric dewpoint at the levels in `pressure`. The first entry should be at
+        the same level as the first `pressure` data point.
 
     Returns
     -------
@@ -539,7 +595,7 @@ def parcel_profile_with_lcl(pressure, temperature, dewpt):
     ambient_temperature : `pint.Quantity`
         The atmospheric temperature values, including the value interpolated to the LCL level
     ambient_dew_point : `pint.Quantity`
-        The atmospheric dew point values, including the value interpolated to the LCL level
+        The atmospheric dewpoint values, including the value interpolated to the LCL level
     profile_temperature : `pint.Quantity`
         The parcel profile temperatures at all of the levels in the returned pressures array,
         including the LCL.
@@ -576,7 +632,7 @@ def _parcel_profile_helper(pressure, temperature, dewpt):
     temp_lower = dry_lapse(press_lower, temperature)
 
     # If the pressure profile doesn't make it to the lcl, we can stop here
-    if _greater_or_close(np.nanmin(pressure), press_lcl.m):
+    if _greater_or_close(np.nanmin(pressure.m), press_lcl.m):
         return (press_lower[:-1], press_lcl, np.array([]) * press_lower.units,
                 temp_lower[:-1], temp_lcl, np.array([]) * temp_lower.units)
 
@@ -645,7 +701,7 @@ def saturation_vapor_pressure(temperature):
     Parameters
     ----------
     temperature : `pint.Quantity`
-        The temperature
+        air temperature
 
     Returns
     -------
@@ -681,14 +737,14 @@ def dewpoint_rh(temperature, rh):
     Parameters
     ----------
     temperature : `pint.Quantity`
-        Air temperature
+        air temperature
     rh : `pint.Quantity`
-        Relative humidity expressed as a ratio in the range 0 < rh <= 1
+        relative humidity expressed as a ratio in the range 0 < rh <= 1
 
     Returns
     -------
     `pint.Quantity`
-        The dew point temperature
+        The dewpoint temperature
 
     See Also
     --------
@@ -714,7 +770,7 @@ def dewpoint(e):
     Returns
     -------
     `pint.Quantity`
-        Dew point temperature
+        dewpoint temperature
 
     See Also
     --------
@@ -789,7 +845,7 @@ def saturation_mixing_ratio(tot_press, temperature):
     tot_press: `pint.Quantity`
         Total atmospheric pressure
     temperature: `pint.Quantity`
-        The temperature
+        air temperature
 
     Returns
     -------
@@ -936,7 +992,7 @@ def virtual_temperature(temperature, mixing, molecular_weight_ratio=mpconsts.eps
     Parameters
     ----------
     temperature: `pint.Quantity`
-        The temperature
+        air temperature
     mixing : `pint.Quantity`
         dimensionless mass mixing ratio
     molecular_weight_ratio : `pint.Quantity` or float, optional
@@ -973,7 +1029,7 @@ def virtual_potential_temperature(pressure, temperature, mixing,
     pressure: `pint.Quantity`
         Total atmospheric pressure
     temperature: `pint.Quantity`
-        The temperature
+        air temperature
     mixing : `pint.Quantity`
         dimensionless mass mixing ratio
     molecular_weight_ratio : `pint.Quantity` or float, optional
@@ -1006,10 +1062,10 @@ def density(pressure, temperature, mixing, molecular_weight_ratio=mpconsts.epsil
 
     Parameters
     ----------
-    temperature: `pint.Quantity`
-        The temperature
     pressure: `pint.Quantity`
         Total atmospheric pressure
+    temperature: `pint.Quantity`
+        air temperature
     mixing : `pint.Quantity`
         dimensionless mass mixing ratio
     molecular_weight_ratio : `pint.Quantity` or float, optional
@@ -1328,21 +1384,21 @@ def cape_cin(pressure, temperature, dewpt, parcel_profile):
     Parameters
     ----------
     pressure : `pint.Quantity`
-        The atmospheric pressure level(s) of interest. The first entry should be the starting
-        point pressure.
+        The atmospheric pressure level(s) of interest, in order from highest to
+        lowest pressure.
     temperature : `pint.Quantity`
         The atmospheric temperature corresponding to pressure.
     dewpt : `pint.Quantity`
-        The atmospheric dew point corresponding to pressure.
+        The atmospheric dewpoint corresponding to pressure.
     parcel_profile : `pint.Quantity`
-        The temperature profile of the parcel
+        The temperature profile of the parcel.
 
     Returns
     -------
     `pint.Quantity`
-        Convective available potential energy (CAPE).
+        Convective Available Potential Energy (CAPE).
     `pint.Quantity`
-        Convective inhibition (CIN).
+        Convective INhibition (CIN).
 
     Notes
     -----
@@ -1397,17 +1453,17 @@ def cape_cin(pressure, temperature, dewpt, parcel_profile):
 
     # CAPE
     # Only use data between the LFC and EL for calculation
-    p_mask = _less_or_close(x, lfc_pressure) & _greater_or_close(x, el_pressure)
-    x_clipped = x[p_mask]
-    y_clipped = y[p_mask]
+    p_mask = _less_or_close(x.m, lfc_pressure) & _greater_or_close(x.m, el_pressure)
+    x_clipped = x[p_mask].magnitude
+    y_clipped = y[p_mask].magnitude
     cape = (mpconsts.Rd
             * (np.trapz(y_clipped, np.log(x_clipped)) * units.degK)).to(units('J/kg'))
 
     # CIN
     # Only use data between the surface and LFC for calculation
-    p_mask = _greater_or_close(x, lfc_pressure)
-    x_clipped = x[p_mask]
-    y_clipped = y[p_mask]
+    p_mask = _greater_or_close(x.m, lfc_pressure)
+    x_clipped = x[p_mask].magnitude
+    y_clipped = y[p_mask].magnitude
     cin = (mpconsts.Rd
            * (np.trapz(y_clipped, np.log(x_clipped)) * units.degK)).to(units('J/kg'))
 
@@ -1436,8 +1492,7 @@ def _find_append_zero_crossings(x, y):
         y values of data
 
     """
-    # Find and append crossings to the data
-    crossings = find_intersections(x[1:], y[1:], np.zeros_like(y[1:]) * y.units)
+    crossings = find_intersections(x[1:], y[1:], np.zeros_like(y[1:]) * y.units, log_x=True)
     x = concatenate((x, crossings[0]))
     y = concatenate((y, crossings[1]))
 
@@ -1447,7 +1502,7 @@ def _find_append_zero_crossings(x, y):
     y = y[sort_idx]
 
     # Remove duplicate data points if there are any
-    keep_idx = np.ediff1d(x, to_end=[1]) > 0
+    keep_idx = np.ediff1d(x.magnitude, to_end=[1]) > 1e-6
     x = x[keep_idx]
     y = y[keep_idx]
     return x, y
@@ -1484,7 +1539,7 @@ def most_unstable_parcel(pressure, temperature, dewpoint, heights=None,
     Returns
     -------
     `pint.Quantity`
-        Pressure, temperature, and dew point of most unstable parcel in the profile.
+        Pressure, temperature, and dewpoint of most unstable parcel in the profile.
     integer
         Index of the most unstable parcel in the given profile
 
@@ -1527,7 +1582,7 @@ def isentropic_interpolation(theta_levels, pressure, temperature, *args, **kwarg
     ----------------
     axis : int, optional
         The axis corresponding to the vertical in the temperature array, defaults to 0.
-    tmpk_out : bool, optional
+    temperature_out : bool, optional
         If true, will calculate temperature and output as the last item in the output list.
         Defaults to False.
     max_iters : int, optional
@@ -1547,6 +1602,9 @@ def isentropic_interpolation(theta_levels, pressure, temperature, *args, **kwarg
     [Ziv1994]_. Any additional arguments are assumed to vary linearly with temperature and will
     be linearly interpolated to the new isentropic levels.
 
+    `isentropic_interpolation` previously accepted `tmpk_out` as an argument. That has been
+    deprecated in 0.11 in favor of `temperature_out` and support will end in 1.0.
+
     See Also
     --------
     potential_temperature
@@ -1564,7 +1622,11 @@ def isentropic_interpolation(theta_levels, pressure, temperature, *args, **kwarg
 
     # Change when Python 2.7 no longer supported
     # Pull out keyword arguments
-    tmpk_out = kwargs.pop('tmpk_out', False)
+    temperature_out = kwargs.get('tmpk_out')
+    if temperature_out is not None:
+        warnings.warn('The use of "tmpk_out" has been deprecated in favor of'
+                      '"temperature_out",', metpyDeprecation)
+    temperature_out = kwargs.pop('temperature_out', temperature_out)
     max_iters = kwargs.pop('max_iters', 50)
     eps = kwargs.pop('eps', 1e-6)
     axis = kwargs.pop('axis', 0)
@@ -1580,7 +1642,7 @@ def isentropic_interpolation(theta_levels, pressure, temperature, *args, **kwarg
     slices = [np.newaxis] * ndim
     slices[axis] = slice(None)
     slices = tuple(slices)
-    pres = np.broadcast_to(pres[slices], temperature.shape) * pres.units
+    pres = np.broadcast_to(pres[slices].magnitude, temperature.shape) * pres.units
 
     # Sort input data
     sort_pres = np.argsort(pres.m, axis=axis)
@@ -1643,18 +1705,15 @@ def isentropic_interpolation(theta_levels, pressure, temperature, *args, **kwarg
     # create list for storing output data
     ret = [isentprs * units.hPa]
 
-    # if tmpk_out = true, calculate temperature and output as last item in list
-    if tmpk_out:
+    # if temperature_out = true, calculate temperature and output as last item in list
+    if temperature_out:
         ret.append((isentlevs_nd / ((mpconsts.P0.m / isentprs) ** ka)) * units.kelvin)
 
     # do an interpolation for each additional argument
     if args:
         others = interpolate_1d(isentlevels, pres_theta.m, *(arr[sorter] for arr in args),
-                                axis=axis)
-        if len(args) > 1:
-            ret.extend(others)
-        else:
-            ret.append(others)
+                                axis=axis, return_list_always=True)
+        ret.extend(others)
 
     return ret
 
@@ -1675,11 +1734,11 @@ def surface_based_cape_cin(pressure, temperature, dewpoint):
     ----------
     pressure : `pint.Quantity`
         Atmospheric pressure profile. The first entry should be the starting
-        (surface) observation.
+        (surface) observation, with the array going from high to low pressure.
     temperature : `pint.Quantity`
-        Temperature profile
+        Temperature profile corresponding to the `pressure` profile.
     dewpoint : `pint.Quantity`
-        Dewpoint profile
+        Dewpoint profile corresponding to the `pressure` profile.
 
     Returns
     -------
@@ -1716,7 +1775,7 @@ def most_unstable_cape_cin(pressure, temperature, dewpoint, **kwargs):
     temperature : `pint.Quantity`
         Temperature profile
     dewpoint : `pint.Quantity`
-        Dewpoint profile
+        Dew point profile
 
     Returns
     -------
@@ -1730,13 +1789,11 @@ def most_unstable_cape_cin(pressure, temperature, dewpoint, **kwargs):
     cape_cin, most_unstable_parcel, parcel_profile
 
     """
-    _, parcel_temperature, parcel_dewpoint, parcel_idx = most_unstable_parcel(pressure,
-                                                                              temperature,
-                                                                              dewpoint,
-                                                                              **kwargs)
-    mu_profile = parcel_profile(pressure[parcel_idx:], parcel_temperature, parcel_dewpoint)
-    return cape_cin(pressure[parcel_idx:], temperature[parcel_idx:],
-                    dewpoint[parcel_idx:], mu_profile)
+    _, _, _, parcel_idx = most_unstable_parcel(pressure, temperature, dewpoint, **kwargs)
+    p, t, td, mu_profile = parcel_profile_with_lcl(pressure[parcel_idx:],
+                                                   temperature[parcel_idx:],
+                                                   dewpoint[parcel_idx:])
+    return cape_cin(p, t, td, mu_profile)
 
 
 @exporter.export
@@ -1772,8 +1829,12 @@ def mixed_parcel(p, temperature, dewpt, parcel_start_pressure=None,
 
     Returns
     -------
-    `pint.Quantity, pint.Quantity, pint.Quantity`
-        The pressure, temperature, and dewpoint of the mixed parcel.
+    `pint.Quantity`
+        The pressure of the mixed parcel
+    `pint.Quantity`
+        The temperature of the mixed parcel
+    `pint.Quantity`
+        The dewpoint of the mixed parcel
 
     """
     # If a parcel starting pressure is not provided, use the surface
@@ -1847,7 +1908,7 @@ def mixed_layer(p, *args, **kwargs):
     ret = []
     for datavar_layer in datavars_layer:
         actual_depth = abs(p_layer[0] - p_layer[-1])
-        ret.append((-1. / actual_depth.m) * np.trapz(datavar_layer, p_layer)
+        ret.append((-1. / actual_depth.m) * np.trapz(datavar_layer.m, p_layer.m)
                    * datavar_layer.units)
     return ret
 
@@ -1870,10 +1931,10 @@ def dry_static_energy(heights, temperature):
 
     Parameters
     ----------
-    heights : array-like
+    heights : `pint.Quantity`
         Atmospheric height
-    temperature : array-like
-        Atmospheric temperature
+    temperature : `pint.Quantity`
+        Air temperature
 
     Returns
     -------
@@ -1892,6 +1953,7 @@ def moist_static_energy(heights, temperature, specific_humidity):
 
     This function will calculate the moist static energy following
     equation 3.72 in [Hobbs2006]_.
+
     Notes
     -----
     .. math::\text{moist static energy} = c_{pd} * T + gz + L_v q
@@ -1902,11 +1964,11 @@ def moist_static_energy(heights, temperature, specific_humidity):
 
     Parameters
     ----------
-    heights : array-like
+    heights : `pint.Quantity`
         Atmospheric height
-    temperature : array-like
-        Atmospheric temperature
-    specific_humidity : array-like
+    temperature : `pint.Quantity`
+        Air temperature
+    specific_humidity : `pint.Quantity`
         Atmospheric specific humidity
 
     Returns
@@ -1989,7 +2051,7 @@ def thickness_hydrostatic(pressure, temperature, **kwargs):
 
     # Take the integral (with unit handling) and return the result in meters
     return (- mpconsts.Rd / mpconsts.g * np.trapz(
-        layer_virttemp.to('K'), x=np.log(layer_p / units.hPa)) * units.K).to('m')
+        layer_virttemp.m_as('K'), x=np.log(layer_p.m_as('hPa'))) * units.K).to('m')
 
 
 @exporter.export
@@ -2062,16 +2124,16 @@ def brunt_vaisala_frequency_squared(heights, potential_temperature, axis=0):
 
     Parameters
     ----------
-    heights : array-like
+    heights : `pint.Quantity`
         One-dimensional profile of atmospheric height
-    potential_temperature : array-like
+    potential_temperature : `pint.Quantity`
         Atmospheric potential temperature
     axis : int, optional
         The axis corresponding to vertical in the potential temperature array, defaults to 0.
 
     Returns
     -------
-    array-like
+    `pint.Quantity`
         The square of the Brunt-Vaisala frequency.
 
     See Also
@@ -2104,16 +2166,16 @@ def brunt_vaisala_frequency(heights, potential_temperature, axis=0):
 
     Parameters
     ----------
-    heights : array-like
+    heights : `pint.Quantity`
         One-dimensional profile of atmospheric height
-    potential_temperature : array-like
+    potential_temperature : `pint.Quantity`
         Atmospheric potential temperature
     axis : int, optional
         The axis corresponding to vertical in the potential temperature array, defaults to 0.
 
     Returns
     -------
-    array-like
+    `pint.Quantity`
         Brunt-Vaisala frequency.
 
     See Also
@@ -2143,16 +2205,16 @@ def brunt_vaisala_period(heights, potential_temperature, axis=0):
 
     Parameters
     ----------
-    heights : array-like
+    heights : `pint.Quantity`
         One-dimensional profile of atmospheric height
-    potential_temperature : array-like
+    potential_temperature : pint.Quantity`
         Atmospheric potential temperature
     axis : int, optional
         The axis corresponding to vertical in the potential temperature array, defaults to 0.
 
     Returns
     -------
-    array-like
+    `pint.Quantity`
         Brunt-Vaisala period.
 
     See Also
@@ -2188,7 +2250,7 @@ def wet_bulb_temperature(pressure, temperature, dewpoint):
 
     Returns
     -------
-    array-like
+    `pint.Quantity`
         Wet-bulb temperature
 
     See Also
@@ -2228,13 +2290,13 @@ def static_stability(pressure, temperature, axis=0):
 
     .. math:: \sigma = -\frac{RT}{p} \frac{\partial \ln \theta}{\partial p}
 
-    This formuala is based on equation 4.3.6 in [Bluestein1992]_.
+    This formula is based on equation 4.3.6 in [Bluestein1992]_.
 
     Parameters
     ----------
-    pressure : array-like
+    pressure : `pint.Quantity`
         Profile of atmospheric pressure
-    temperature : array-like
+    temperature : `pint.Quantity`
         Profile of temperature
     axis : int, optional
         The axis corresponding to vertical in the pressure and temperature arrays, defaults
@@ -2242,13 +2304,13 @@ def static_stability(pressure, temperature, axis=0):
 
     Returns
     -------
-    array-like
+    `pint.Quantity`
         The profile of static stability.
 
     """
     theta = potential_temperature(pressure, temperature)
 
-    return - mpconsts.Rd * temperature / pressure * first_derivative(np.log(theta / units.K),
+    return - mpconsts.Rd * temperature / pressure * first_derivative(np.log(theta.m_as('K')),
                                                                      x=pressure, axis=axis)
 
 
@@ -2270,7 +2332,7 @@ def dewpoint_from_specific_humidity(specific_humidity, temperature, pressure):
     Returns
     -------
     `pint.Quantity`
-        Dewpoint temperature
+        Dew point temperature
 
     See Also
     --------
@@ -2294,7 +2356,7 @@ def vertical_velocity_pressure(w, pressure, temperature, mixing=0):
     assuming hydrostatic conditions on the synoptic scale.
     By Equation 7.33 in [Hobbs2006]_,
 
-    .. math: \omega \simeq -\rho g w
+    .. math:: \omega \simeq -\rho g w
 
     Density (:math:`\rho`) is calculated using the :func:`density` function,
     from the given pressure and temperature. If `mixing` is given, the virtual
@@ -2336,11 +2398,11 @@ def vertical_velocity(omega, pressure, temperature, mixing=0):
     :math:`\left(w = \frac{Dz}{Dt}\right)` assuming hydrostatic conditions on
     the synoptic scale. By Equation 7.33 in [Hobbs2006]_,
 
-    .. math: \omega \simeq -\rho g w
+    .. math:: \omega \simeq -\rho g w
 
     so that
 
-    .. math w \simeq \frac{- \omega}{\rho g}
+    .. math:: w \simeq \frac{- \omega}{\rho g}
 
     Density (:math:`\rho`) is calculated using the :func:`density` function,
     from the given pressure and temperature. If `mixing` is given, the virtual
@@ -2369,3 +2431,31 @@ def vertical_velocity(omega, pressure, temperature, mixing=0):
     """
     rho = density(pressure, temperature, mixing)
     return (omega / (- mpconsts.g * rho)).to('m/s')
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[temperature]', '[pressure]')
+def specific_humidity_from_dewpoint(dewpoint, pressure):
+    r"""Calculate the specific humidity from the dewpoint temperature and pressure.
+
+    Parameters
+    ----------
+    dewpoint: `pint.Quantity`
+        dewpoint temperature
+
+    pressure: `pint.Quantity`
+        pressure
+
+    Returns
+    -------
+    `pint.Quantity`
+        Specific humidity
+
+    See Also
+    --------
+    mixing_ratio, saturation_mixing_ratio
+
+    """
+    mixing_ratio = saturation_mixing_ratio(pressure, dewpoint)
+    return specific_humidity_from_mixing_ratio(mixing_ratio)
