@@ -1,4 +1,4 @@
-# Copyright (c) 2016,2017,2018 MetPy Developers.
+# Copyright (c) 2016,2017,2018,2019 MetPy Developers.
 # Distributed under the terms of the BSD 3-Clause License.
 # SPDX-License-Identifier: BSD-3-Clause
 """Contains a collection of generally useful calculation tools."""
@@ -20,25 +20,30 @@ from scipy.spatial import cKDTree
 import xarray as xr
 
 from . import height_to_pressure_std, pressure_to_height_std
-from ..cbook import broadcast_indices
+from ..cbook import broadcast_indices, result_type
 from ..deprecation import deprecated, metpyDeprecation
 from ..interpolate.one_dimension import interpolate_1d, interpolate_nans_1d, log_interpolate_1d
 from ..package_tools import Exporter
 from ..units import atleast_1d, check_units, concatenate, diff, units
-from ..xarray import CFConventionHandler, preprocess_xarray
+from ..xarray import check_axis, preprocess_xarray
 
 exporter = Exporter(globals())
 
-DIR_STRS = [
+UND = 'UND'
+UND_ANGLE = -999.
+DIR_STRS = (
     'N', 'NNE', 'NE', 'ENE',
     'E', 'ESE', 'SE', 'SSE',
     'S', 'SSW', 'SW', 'WSW',
-    'W', 'WNW', 'NW', 'NNW'
-]
+    'W', 'WNW', 'NW', 'NNW',
+    UND
+)  # note the order matters!
 
+MAX_DEGREE_ANGLE = 360 * units.degree
 BASE_DEGREE_MULTIPLIER = 22.5 * units.degree
 
 DIR_DICT = {dir_str: i * BASE_DEGREE_MULTIPLIER for i, dir_str in enumerate(DIR_STRS)}
+DIR_DICT[UND] = np.nan
 
 
 @exporter.export
@@ -98,7 +103,7 @@ def nearest_intersection_idx(a, b):
 @exporter.export
 @preprocess_xarray
 @units.wraps(('=A', '=B'), ('=A', '=B', '=B'))
-def find_intersections(x, a, b, direction='all'):
+def find_intersections(x, a, b, direction='all', log_x=False):
     """Calculate the best estimate of intersection.
 
     Calculates the best estimates of the intersection of two y-value
@@ -115,6 +120,9 @@ def find_intersections(x, a, b, direction='all'):
     direction : string, optional
         specifies direction of crossing. 'all', 'increasing' (a becoming greater than b),
         or 'decreasing' (b becoming greater than a). Defaults to 'all'.
+    log_x : bool, optional
+        Use logarithmic interpolation along the `x` axis (i.e. for finding intersections
+        in pressure coordinates). Default is False.
 
     Returns
     -------
@@ -122,6 +130,10 @@ def find_intersections(x, a, b, direction='all'):
         intersections of the lines.
 
     """
+    # Change x to logarithmic if log_x=True
+    if log_x is True:
+        x = np.log(x)
+
     # Find the index of the points just before the intersection(s)
     nearest_idx = nearest_intersection_idx(a, b)
     next_idx = nearest_idx + 1
@@ -157,6 +169,10 @@ def find_intersections(x, a, b, direction='all'):
     if len(intersect_x) == 0:
         return intersect_x, intersect_y
 
+    # Return x to linear if log_x is True
+    if log_x is True:
+        intersect_x = np.exp(intersect_x)
+
     # Check for duplicates
     duplicate_mask = (np.ediff1d(intersect_x, to_end=1) != 0)
 
@@ -248,14 +264,15 @@ def reduce_point_density(points, radius, priority=None):
     data), returning a mask that can be used to select the points from one or more arrays
     (e.g. arrays of temperature and dew point). The points selected can be controlled by
     providing an array of ``priority`` values (e.g. rainfall totals to ensure that
-    stations with higher precipitation remain in the mask).
+    stations with higher precipitation remain in the mask). The points and radius can be
+    specified with units. If none are provided, meters are assumed.
 
     Parameters
     ----------
     points : (N, K) array-like
         N locations of the points in K dimensional space
-    radius : float
-        minimum radius allowed between points
+    radius : `pint.Quantity` or float
+        Minimum radius allowed between points. If units are not provided, meters is assumed.
     priority : (N, K) array-like, optional
         If given, this should have the same shape as ``points``; these values will
         be used to control selection priority for points.
@@ -268,12 +285,19 @@ def reduce_point_density(points, radius, priority=None):
     Examples
     --------
     >>> metpy.calc.reduce_point_density(np.array([1, 2, 3]), 1.)
-    array([ True, False,  True])
+    array([ True, False, True])
     >>> metpy.calc.reduce_point_density(np.array([1, 2, 3]), 1.,
     ... priority=np.array([0.1, 0.9, 0.3]))
-    array([False,  True, False])
+    array([False, True, False])
 
     """
+    # Handle input with units. Assume meters if units are not specified
+    if hasattr(radius, 'units'):
+        radius = radius.to('m').m
+
+    if hasattr(points, 'units'):
+        points = points.to('m').m
+
     # Handle 1D input
     if points.ndim < 2:
         points = points.reshape(-1, 1)
@@ -311,8 +335,8 @@ def _get_bound_pressure_height(pressure, bound, heights=None, interpolate=True):
     """Calculate the bounding pressure and height in a layer.
 
     Given pressure, optional heights, and a bound, return either the closest pressure/height
-    or interpolated pressure/height. If no heights are provided, a standard atmosphere is
-    assumed.
+    or interpolated pressure/height. If no heights are provided, a standard atmosphere
+    ([NOAA1976]_) is assumed.
 
     Parameters
     ----------
@@ -379,8 +403,9 @@ def _get_bound_pressure_height(pressure, bound, heights=None, interpolate=True):
                     # Need to cast back to the input type since interp (up to at least numpy
                     # 1.13 always returns float64. This can cause upstream users problems,
                     # resulting in something like np.append() to upcast.
-                    bound_pressure = np.interp(np.atleast_1d(bound), heights,
-                                               pressure).astype(bound.dtype) * pressure.units
+                    bound_pressure = (np.interp(np.atleast_1d(bound.m), heights.m,
+                                                pressure.m).astype(result_type(bound))
+                                      * pressure.units)
                 else:
                     idx = (np.abs(heights - bound)).argmin()
                     bound_pressure = pressure[idx]
@@ -400,13 +425,15 @@ def _get_bound_pressure_height(pressure, bound, heights=None, interpolate=True):
         raise ValueError('Bound must be specified in units of length or pressure.')
 
     # If the bound is out of the range of the data, we shouldn't extrapolate
-    if not (_greater_or_close(bound_pressure, np.nanmin(pressure) * pressure.units)
-            and _less_or_close(bound_pressure, np.nanmax(pressure) * pressure.units)):
+    if not (_greater_or_close(bound_pressure, np.nanmin(pressure.m) * pressure.units)
+            and _less_or_close(bound_pressure, np.nanmax(pressure.m) * pressure.units)):
         raise ValueError('Specified bound is outside pressure range.')
-    if heights is not None:
-        if not (_less_or_close(bound_height, np.nanmax(heights) * heights.units)
-                and _greater_or_close(bound_height, np.nanmin(heights) * heights.units)):
-            raise ValueError('Specified bound is outside height range.')
+    if heights is not None and not (_less_or_close(bound_height,
+                                                   np.nanmax(heights.m) * heights.units)
+                                    and _greater_or_close(bound_height,
+                                                          np.nanmin(heights.m)
+                                                          * heights.units)):
+        raise ValueError('Specified bound is outside height range.')
 
     return bound_pressure, bound_height
 
@@ -426,7 +453,7 @@ def get_layer_heights(heights, depth, *args, **kwargs):
         Atmospheric heights
     depth : `pint.Quantity`
         The thickness of the layer
-    *args : array-like
+    args : array-like
         Atmospheric variable(s) measured at the given pressures
     bottom : `pint.Quantity`, optional
         The bottom of the layer
@@ -482,9 +509,9 @@ def get_layer_heights(heights, depth, *args, **kwargs):
     if interpolate:
         # If we don't have the bottom or top requested, append them
         if top not in heights_interp:
-            heights_interp = np.sort(np.append(heights_interp, top)) * heights.units
+            heights_interp = np.sort(np.append(heights_interp.m, top.m)) * heights.units
         if bottom not in heights_interp:
-            heights_interp = np.sort(np.append(heights_interp, bottom)) * heights.units
+            heights_interp = np.sort(np.append(heights_interp.m, bottom.m)) * heights.units
 
     ret.append(heights_interp)
 
@@ -519,11 +546,11 @@ def get_layer(pressure, *args, **kwargs):
     ----------
     pressure : array-like
         Atmospheric pressure profile
-    *args : array-like
+    args : array-like
         Atmospheric variable(s) measured at the given pressures
     heights: array-like, optional
         Atmospheric heights corresponding to the given pressures. Defaults to using
-        heights calculated from ``p`` assuming a standard atmosphere.
+        heights calculated from ``p`` assuming a standard atmosphere [NOAA1976]_.
     bottom : `pint.Quantity`, optional
         The bottom of the layer as a pressure or height above the surface pressure. Defaults
         to the highest pressure or lowest height given.
@@ -557,7 +584,7 @@ def get_layer(pressure, *args, **kwargs):
 
     # If the bottom is not specified, make it the surface pressure
     if bottom is None:
-        bottom = np.nanmax(pressure) * pressure.units
+        bottom = np.nanmax(pressure.m) * pressure.units
 
     bottom_pressure, bottom_height = _get_bound_pressure_height(pressure, bottom,
                                                                 heights=heights,
@@ -589,9 +616,9 @@ def get_layer(pressure, *args, **kwargs):
     if interpolate:
         # If we don't have the bottom or top requested, append them
         if not np.any(np.isclose(top_pressure, p_interp)):
-            p_interp = np.sort(np.append(p_interp, top_pressure)) * pressure.units
+            p_interp = np.sort(np.append(p_interp.m, top_pressure.m)) * pressure.units
         if not np.any(np.isclose(bottom_pressure, p_interp)):
-            p_interp = np.sort(np.append(p_interp, bottom_pressure)) * pressure.units
+            p_interp = np.sort(np.append(p_interp.m, bottom_pressure.m)) * pressure.units
 
     ret.append(p_interp[::-1])
 
@@ -773,47 +800,6 @@ def _less_or_close(a, value, **kwargs):
     return (a < value) | np.isclose(a, value, **kwargs)
 
 
-@deprecated('0.8', addendum=' This function has been replaced by the signed delta distance'
-                            'calculation lat_lon_grid_deltas and will be removed in MetPy'
-                            ' 0.11.',
-            pending=False)
-@exporter.export
-@preprocess_xarray
-def lat_lon_grid_spacing(longitude, latitude, **kwargs):
-    r"""Calculate the distance between grid points that are in a latitude/longitude format.
-
-    Calculate the distance between grid points when the grid spacing is defined by
-    delta lat/lon rather than delta x/y
-
-    Parameters
-    ----------
-    longitude : array_like
-        array of longitudes defining the grid
-    latitude : array_like
-        array of latitudes defining the grid
-    kwargs
-        Other keyword arguments to pass to :class:`~pyproj.Geod`
-
-    Returns
-    -------
-     dx, dy: 2D arrays of distances between grid points in the x and y direction
-
-    Notes
-    -----
-    Accepts, 1D or 2D arrays for latitude and longitude
-    Assumes [Y, X] for 2D arrays
-
-    .. deprecated:: 0.8.0
-        Function has been replaced with the signed delta distance calculation
-        `lat_lon_grid_deltas` and will be removed from MetPy in 0.11.0.
-
-    """
-    # Use the absolute value of the signed function replacing this
-    dx, dy = lat_lon_grid_deltas(longitude, latitude, **kwargs)
-
-    return np.abs(dx), np.abs(dy)
-
-
 @exporter.export
 @preprocess_xarray
 def lat_lon_grid_deltas(longitude, latitude, **kwargs):
@@ -925,13 +911,13 @@ def xarray_derivative_wrap(func):
             # Initialize new kwargs with the axis number
             new_kwargs = {'axis': f.get_axis_num(axis)}
 
-            if f[axis].attrs.get('_metpy_axis') == 'T':
+            if check_axis(f[axis], 'time'):
                 # Time coordinate, need to convert to seconds from datetimes
                 new_kwargs['x'] = f[axis].metpy.as_timestamp().metpy.unit_array
-            elif CFConventionHandler.check_axis(f[axis], 'lon'):
+            elif check_axis(f[axis], 'lon'):
                 # Longitude coordinate, need to get grid deltas
                 new_kwargs['delta'], _ = grid_deltas_from_dataarray(f)
-            elif CFConventionHandler.check_axis(f[axis], 'lat'):
+            elif check_axis(f[axis], 'lat'):
                 # Latitude coordinate, need to get grid deltas
                 _, new_kwargs['delta'] = grid_deltas_from_dataarray(f)
             else:
@@ -1321,7 +1307,7 @@ def _process_deriv_args(f, kwargs):
             diff_size[axis] -= 1
             delta_units = getattr(delta, 'units', None)
             delta = np.broadcast_to(delta, diff_size, subok=True)
-            if delta_units is not None:
+            if not hasattr(delta, 'units') and delta_units is not None:
                 delta = delta * delta_units
         else:
             delta = _broadcast_to_axis(delta, axis, n)
@@ -1344,25 +1330,38 @@ def parse_angle(input_dir):
 
     Parameters
     ----------
-    input_dir : string or array-like strings
+    input_dir : string or array-like
         Directional text such as west, [south-west, ne], etc
 
     Returns
     -------
-    angle
+    `pint.Quantity`
         The angle in degrees
 
     """
     if isinstance(input_dir, str):
         # abb_dirs = abbrieviated directions
-        abb_dirs = [_abbrieviate_direction(input_dir)]
-    elif isinstance(input_dir, list):
-        input_dir_str = ','.join(input_dir)
+        abb_dirs = _clean_direction([_abbrieviate_direction(input_dir)])
+    elif hasattr(input_dir, '__len__'):  # handle np.array, pd.Series, list, and array-like
+        input_dir_str = ','.join(_clean_direction(input_dir, preprocess=True))
         abb_dir_str = _abbrieviate_direction(input_dir_str)
-        abb_dirs = abb_dir_str.split(',')
+        abb_dirs = _clean_direction(abb_dir_str.split(','))
+    else:  # handle unrecognizable scalar
+        return np.nan
+
     return itemgetter(*abb_dirs)(DIR_DICT)
 
 
+def _clean_direction(dir_list, preprocess=False):
+    """Handle None if preprocess, else handles anything not in DIR_STRS."""
+    if preprocess:  # primarily to remove None from list so ','.join works
+        return [UND if not isinstance(the_dir, str) else the_dir
+                for the_dir in dir_list]
+    else:  # remove extraneous abbrieviated directions
+        return [UND if the_dir not in DIR_STRS else the_dir
+                for the_dir in dir_list]
+
+
 def _abbrieviate_direction(ext_dir_str):
     """Convert extended (non-abbrievated) directions to abbrieviation."""
     return (ext_dir_str
@@ -1375,3 +1374,109 @@ def _abbrieviate_direction(ext_dir_str):
             .replace('SOUTH', 'S')
             .replace('WEST', 'W')
             )
+
+
+@exporter.export
+@preprocess_xarray
+def angle_to_direction(input_angle, full=False, level=3):
+    """Convert the meteorological angle to directional text.
+
+    Works for angles greater than or equal to 360 (360 -> N | 405 -> NE)
+    and rounds to the nearest angle (355 -> N | 404 -> NNE)
+
+    Parameters
+    ----------
+    input_angle : numeric or array-like numeric
+        Angles such as 0, 25, 45, 360, 410, etc
+    full : boolean
+        True returns full text (South), False returns abbrieviated text (S)
+    level : int
+        Level of detail (3 = N/NNE/NE/ENE/E... 2 = N/NE/E/SE... 1 = N/E/S/W)
+
+    Returns
+    -------
+    direction
+        The directional text
+
+    """
+    try:  # strip units temporarily
+        origin_units = input_angle.units
+        input_angle = input_angle.m
+    except AttributeError:  # no units associated
+        origin_units = units.degree
+
+    if not hasattr(input_angle, '__len__') or isinstance(input_angle, str):
+        input_angle = [input_angle]
+        scalar = True
+    else:
+        scalar = False
+
+    # clean any numeric strings, negatives, and None
+    # does not handle strings with alphabet
+    input_angle = np.array(input_angle).astype(float)
+    with np.errstate(invalid='ignore'):  # warns about the np.nan
+        input_angle[np.where(input_angle < 0)] = np.nan
+
+    input_angle = input_angle * origin_units
+
+    # normalizer used for angles > 360 degree to normalize between 0 - 360
+    normalizer = np.array(input_angle.m / MAX_DEGREE_ANGLE.m, dtype=int)
+    norm_angles = abs(input_angle - MAX_DEGREE_ANGLE * normalizer)
+
+    if level == 3:
+        nskip = 1
+    elif level == 2:
+        nskip = 2
+    elif level == 1:
+        nskip = 4
+    else:
+        err_msg = 'Level of complexity cannot be less than 1 or greater than 3!'
+        raise ValueError(err_msg)
+
+    angle_dict = {i * BASE_DEGREE_MULTIPLIER.m * nskip: dir_str
+                  for i, dir_str in enumerate(DIR_STRS[::nskip])}
+    angle_dict[MAX_DEGREE_ANGLE.m] = 'N'  # handle edge case of 360.
+    angle_dict[UND_ANGLE] = UND
+
+    # round to the nearest angles for dict lookup
+    # 0.001 is subtracted so there's an equal number of dir_str from
+    # np.arange(0, 360, 22.5), or else some dir_str will be preferred
+
+    # without the 0.001, level=2 would yield:
+    # ['N', 'N', 'NE', 'E', 'E', 'E', 'SE', 'S', 'S',
+    #  'S', 'SW', 'W', 'W', 'W', 'NW', 'N']
+
+    # with the -0.001, level=2 would yield:
+    # ['N', 'N', 'NE', 'NE', 'E', 'E', 'SE', 'SE',
+    #  'S', 'S', 'SW', 'SW', 'W', 'W', 'NW', 'NW']
+
+    multiplier = np.round(
+        (norm_angles / BASE_DEGREE_MULTIPLIER / nskip) - 0.001).m
+    round_angles = (multiplier * BASE_DEGREE_MULTIPLIER.m * nskip)
+    round_angles[np.where(np.isnan(round_angles))] = UND_ANGLE
+
+    dir_str_arr = itemgetter(*round_angles)(angle_dict)  # for array
+    if full:
+        dir_str_arr = ','.join(dir_str_arr)
+        dir_str_arr = _unabbrieviate_direction(dir_str_arr)
+        if not scalar:
+            dir_str = dir_str_arr.split(',')
+        else:
+            dir_str = dir_str_arr.replace(',', ' ')
+    else:
+        dir_str = dir_str_arr
+
+    return dir_str
+
+
+def _unabbrieviate_direction(abb_dir_str):
+    """Convert abbrieviated directions to non-abbrieviated direction."""
+    return (abb_dir_str
+            .upper()
+            .replace(UND, 'Undefined ')
+            .replace('N', 'North ')
+            .replace('E', 'East ')
+            .replace('S', 'South ')
+            .replace('W', 'West ')
+            .replace(' ,', ',')
+            ).strip()
