@@ -28,12 +28,13 @@ def get_upper_air_data(date, station):
     """Get upper air observations from the test data cache.
 
     Parameters
-     ----------
+    ----------
     time : datetime
           The date and time of the desired observation.
     station : str
          The three letter ICAO identifier of the station for which data should be
          downloaded.
+
     Returns
     -------
         dict : upper air data
@@ -135,11 +136,8 @@ def check_and_drop_units(actual, desired):
 
 def assert_nan(value, units):
     """Check for nan with proper units."""
-    if not np.isnan(value):
-        pytest.fail('{} is not np.nan'.format(value))
-
-    check_and_drop_units(value, np.nan * units)
-    return True
+    value, _ = check_and_drop_units(value, np.nan * units)
+    assert np.isnan(value)
 
 
 def assert_almost_equal(actual, desired, decimal=7):
@@ -199,11 +197,12 @@ def patch_round(monkeypatch):
     monkeypatch.setitem(__builtins__, 'round', np.round)
 
 
-def ignore_deprecation(func):
+def check_and_silence_deprecation(func):
     """Decorate a function to swallow metpy deprecation warnings, making sure they are present.
 
-    This should be used on deprecation function tests to make sure the deprecation warnings
-    are not failing the tests, but still allow testing of those functions.
+    This should be used on deprecated function tests to make sure the deprecation warnings
+    are not printing in the tests, but checks that the warning is present and makes sure
+    the function still works as intended.
     """
     @functools.wraps(func)
     def wrapper(*args, **kwargs):
