@@ -156,13 +156,14 @@ def diff(x, **kwargs):
     numpy.diff
 
     """
-    ret = np.diff(x, **kwargs)
     if hasattr(x, 'units'):
+        ret = np.diff(x.magnitude, **kwargs)
         # Can't just use units because of how things like temperature work
         it = x.flat
         true_units = (next(it) - next(it)).units
-        ret = ret * true_units
-    return ret
+        return ret * true_units
+    else:
+        return np.diff(x, **kwargs)
 
 
 def atleast_1d(*arrs):
@@ -228,17 +229,18 @@ def masked_array(data, data_units=None, **kwargs):
 
     This is a thin wrapper around :func:`numpy.ma.masked_array` that ensures that
     units are properly attached to the result (otherwise units are silently lost). Units
-    are taken from the ``units`` argument, or if this is ``None``, the units on ``data``
+    are taken from the ``data_units`` argument, or if this is ``None``, the units on ``data``
     are used.
 
     Parameters
     ----------
     data : array_like
-        The source data. If ``units`` is `None`, this should be a `pint.Quantity` with
+        The source data. If ``data_units`` is `None`, this should be a `pint.Quantity` with
         the desired units.
-    data_units : str or `pint.Unit`
+    data_units : str or `pint.Unit`, optional
         The units for the resulting `pint.Quantity`
-    **kwargs : Arbitrary keyword arguments passed to `numpy.ma.masked_array`
+    kwargs
+        Arbitrary keyword arguments passed to `numpy.ma.masked_array`, optional
 
     Returns
     -------
@@ -325,17 +327,14 @@ try:
 except (AttributeError, RuntimeError, ImportError):  # Pint's not available, try our own
     import matplotlib.units as munits
 
-    # Inheriting from object fixes the fact that matplotlib 1.4 doesn't
-    # TODO: Remove object when we drop support for matplotlib 1.4
-    class PintAxisInfo(munits.AxisInfo, object):
+    class PintAxisInfo(munits.AxisInfo):
         """Support default axis and tick labeling and default limits."""
 
         def __init__(self, units):
             """Set the default label to the pretty-print of the unit."""
             super(PintAxisInfo, self).__init__(label='{:P}'.format(units))
 
-    # TODO: Remove object when we drop support for matplotlib 1.4
-    class PintConverter(munits.ConversionInterface, object):
+    class PintConverter(munits.ConversionInterface):
         """Implement support for pint within matplotlib's unit conversion framework."""
 
         def __init__(self, registry):
